#include <muses_72323.h>
#include <bitBangedSPI.h>

bitBangedSPI bbSPI (11, 12, 13); // MOSI, MISO, SCK
#define SS    10  // slave select

// Four Muses chip adresses
#define CHIPADDRESS 0

// static const SPISettings s_muses_spi_settings(250000, MSBFIRST, SPI_MODE0);
muses_72323_command_t command;

bool link_channels  = false;
bool use_soft_step  = true;
bool use_zero_cross = true;
muses_72323_zero_window_t zero_window = 0;
muses_72323_soft_step_clock_divider_t clock_divider = 0;
muses_72323_soft_step_clock_source_t  soft_step_clock = MUSES_72323_SOFT_STEP_CLOCK_INTERNAL;

// Function declarations
void volume	     (uint8_t chip_address, uint8_t volume_left, uint8_t volume_right);
void attenuation (uint8_t chip_address, uint8_t channel, uint16_t attenuation);
void mute        (uint8_t chip_address, uint8_t channel);
void gain        (uint8_t chip_address, uint8_t gain_left, uint8_t gain_right);

void musesBegin(void);
void musesConfigure(uint8_t chip_address);
void musesSend(muses_72323_command_t command);

// Volume: 0 to 100
void volume	(uint8_t chip_address, uint8_t volume_left, uint8_t volume_right) {
  attenuation(chip_address, 0, (uint16_t)(MUSES_72323_MAX_ATTENUATION * (100 - volume_left) / 100));
  attenuation(chip_address, 1, (uint16_t)(MUSES_72323_MAX_ATTENUATION * (100 - volume_right) / 100));
}

// dB: 0 to -111.75
void dezibel(uint8_t chip_address, float dB_left, float dB_right) {
  attenuation(chip_address, 0, (uint16_t)(-dB_left / 0.25f));
  attenuation(chip_address, 1, (uint16_t)(-dB_right / 0.25f));
}

// Attenuation: 0 to 447 (447 = full attenuation = silent)
void attenuation (uint8_t chip_address, uint8_t channel, uint16_t attenuation) {
  attenuation = constrain ( attenuation, 0, MUSES_72323_MAX_ATTENUATION);
  muses_72323_set_volume(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_t)channel, (muses_72323_attenuation_t)attenuation, use_soft_step);
  musesSend(command);
  // Serial.printf("chip %d \t channel %d \t attenuation %d \n", chip_address, channel, attenuation);
}

// GAIN: 0 to 7
void gain (uint8_t chip_address, uint8_t gain_left, uint8_t gain_right) {
  gain_left  = constrain ( gain_left,  0, MUSES_72323_MAX_CHANNEL_GAIN);
  gain_right = constrain ( gain_right, 0, MUSES_72323_MAX_CHANNEL_GAIN);
  muses_72323_set_gain(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_gain_t)gain_left, (muses_72323_channel_gain_t)gain_right, link_channels, use_zero_cross);
  musesSend(command);
}

void mute (uint8_t chip_address, uint8_t channel) {
  muses_72323_mute(&command, (muses_72323_chip_address_t)chip_address, (muses_72323_channel_t)channel, use_zero_cross);
  musesSend(command);
}

void musesBegin(void) {
  pinMode(SS, OUTPUT);
  digitalWrite(SS, HIGH);
  bbSPI.begin();
  musesConfigure(CHIPADDRESS);
}

void musesConfigure(uint8_t chip_address) {
  muses_72323_configure(&command, (muses_72323_chip_address_t)chip_address, zero_window, clock_divider, soft_step_clock);
  musesSend(command);
}

void musesSend(muses_72323_command_t command) {
  /*
    Serial.print(highByte(command),BIN);
    Serial.print("\t");
    Serial.println(lowByte(command),BIN);
  */
  digitalWrite(SS, LOW);
  bbSPI.transfer(highByte(command));
  bbSPI.transfer(lowByte(command));
  digitalWrite(SS, HIGH);
}

#if defined(KINETISK)
extern "C" {
  int _write(int file, char* ptr, int len)
  {
    ((class Print*)file)->write((uint8_t*)ptr, len);
    return len;
  }
}
#endif
